{*******************************************************************}
{    PColor 1.3.                                                    }
{    Draw 2D data - flat, interp, isoline.                          }
{     Filatkin, Andrey. 1999                                       }
{    Last modified - 21 February 1999                               }
{*******************************************************************}
{    Credits:                                                       }
{      Thank to                                                     }
{      Agustin (Qxo) Villena (avillena@dcc.uchile.cl)               }
{      and John Biddiscombe (J.Biddiscombe@rl.ac.uk)                }
{      for TDIBPaintBox and TDIBSurface.                            }
{      TDIBWork is reduced version TDIBSurface.                     }
{*******************************************************************}
{  History:                                                         }
{   1.1 : Other  CreateContour.                                     }
{   1.2 : Faster drawing. DIB now is used.                          }
{   1.3 : The orientation of the figure is changed. The element of  }
{       array [0,0] now is represented in the left bottom corner    }
{       (earlier it was in the left top corner).                    }
{       Added : title, zoom, PosImageToPosLog.                      }
{*******************************************************************}
unit Pcolor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls;

const NumbCol = 64;
      LeftMargin = 2;
      TopMargin  = 2;
      RightMargin = 2;
      BottomMargin = 2;
      MaxZoom = 1e6;

type
  PArray = ^TArray;
  TArray = array of array of double;
  TColorMaps = (cmLine,cmSpectr,cmRandom);
  TDrawTypes = (dtFlat,dtInterp,dtContour);

TLogPaletteDIB = record
  palVersion: Word;
  palNumEntries: Word;
  palEntry: array[0..255] of TPaletteEntry;
end;

  TDIBWork = class
  private
    FBits       : Pointer;
    DWordWidth  : integer;
    FHandlePal  : HPALETTE;
    FHandle     : HDC;
    FOldPalette : HPALETTE;
    LogPaletteDIB  : TLogPaletteDIB;
    BitmapInfo : TBitmapInfo;
    hDIB       : HBITMAP;
    OldBitmap  : HBITMAP;
    FWidth : integer;
    FHeight : integer;
    constructor PaletteCreate;
    procedure PaletteChange(LogPal : TLogPaletteDIB);
    function GetPixel(X, Y: Integer): byte;
    procedure SetPixel(X, Y: Integer; Value: byte);
  public
    constructor Create(AWidth, AHeigth : integer);
    destructor Destroy; override;
    procedure SetPalette(LogPal : TLogPaletteDIB);
    procedure Paint(destDC : hDC; ALeft,ATop : Integer); virtual;
    procedure Resize(Width,Height : integer);
    property Pixels[X,Y : integer] : byte read GetPixel write SetPixel;
    property Handle : HDC read FHandle;
    property Width : Integer read FWidth;
    property Height : Integer read FHeight;
  end;

  TTitle = class(TPersistent)
  private
    FAlignment: TAlignment;
    FFont : TFont;
    FText : TCaption;
    FVisible : Boolean;
    FOnChange: TNotifyEvent;
    procedure SetAlignment(Value: TAlignment);
    procedure SetFont(Value: TFont);
    procedure SetText(const Value: TCaption);
    procedure SetVisible(Value: Boolean);
  protected
  public
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Value: TPersistent); override;
    procedure Changed;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment default taCenter;
    property Font : TFont read FFont write SetFont;
    property Text : TCaption read FText write SetText;
    property Visible : Boolean read FVisible write SetVisible default True;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TAxes = class(TPersistent)
  private
    FXMin, FXMax : Extended;
    FYMin, FYMax : Extended;
    FFont : TFont;
    FVisible : Boolean;
    FOnChange: TNotifyEvent;
    procedure SetFont(Value: TFont);
    procedure SetVisible(Value: Boolean);
  protected
  public
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Value: TPersistent); override;
    procedure Changed;
    property XMax : Extended read FXMax;
    property XMin : Extended read FXMin;
    property YMax : Extended read FYMax;
    property YMin : Extended read FYMin;
  published
    property Font : TFont read FFont write SetFont;
    property Visible : Boolean read FVisible write SetVisible default True;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

//  TPColor = class(TGraphicControl)
  TPColor = class(TCustomControl)
  private
    { Private declarations }
    FAbout : String;
    FOnPaint: TNotifyEvent;
    FOnZoom : TNotifyEvent;
    FPCWidth: Integer;
    FPCHeight: Integer;
    FImageLeft : Integer;
    FImageTop : Integer;
    FImageWidth : Integer;
    FImageHeight : Integer;
    FDrawFacet : boolean;
    FColorGrid : TColor;
    FColorFon : TColor;
    FColorMinValue : TColor;
    FColorMaxValue : TColor;
    FColorMap : TColorMaps;
    FTypeImage : TDrawTypes;
    FColorImageFon : TColor;
    FContourLines : Integer;
    TempCursor : TCursor;
    FDIB : TDIBWork;
    FLogPalette  : TLogPaletteDIB;
    NumberX : Integer;
    NumberY : Integer;
    FGraph : TArray;
    FMouseOn: Boolean;
    Origin : TPoint;
    MovePt : TPoint;
    FOnMouseDown : TMouseEvent;
    FOnMouseMove: TMouseMoveEvent;
    FOnMouseUp: TMouseEvent;
    Drawing : Boolean;
    ChangeGraph : Boolean;

    FAxes : TAxes;

    FTitle : TTitle;
    TitleLeft, TitleTop, TitleWidth, TitleHeight : Integer;
    TitleVis : Boolean;

    FLogLeft : Double;
    FLogTop : Double;
    FLogRight : Double;
    FLogBottom : Double;
    FLogCentreX : Double;
    FLogCentreY : Double;
    FHorZoom : Double;
    FVerZoom : Double;

    procedure CreateImage;
    procedure CreatePaletta;
    procedure CreateFlat;
    procedure CreateInterp;
    procedure CreateContour;
    procedure SetAbout(AAbout : String);
    procedure SetColorFon(AColor : TColor);
    procedure SetColorImageFon(AColor : TColor);
    procedure SetColorMaxValue(AColor : TColor);
    procedure SetColorMinValue(AColor : TColor);
    procedure SetColorGrid(AColorGrid : TColor);
    procedure SetDrawFacet(ADrawFacet : boolean);
    procedure SetColorMap(AColorMap : TColorMaps);
    procedure SetTypeImage(ATypeImage : TDrawTypes);
    procedure SetContourLines(AContourLines : Integer);
    procedure SetMouseOn(const Value: Boolean);
    procedure Sizes;
    function GetGraph(X, Y: Integer): Double;
    procedure SetGraph(X, Y: Integer; Value: Double);

    procedure SetAxes(Value: TAxes);

    procedure SetTitle(Value: TTitle);
    procedure TitleChanged(Sender: TObject);
    procedure DoDrawText(var Rect: TRect; Flags: Longint);

    function LogXToArr(Value : Double) : Double;
    function LogYToArr(Value : Double) : Double;
    procedure SetLogLeft(Value : Double);
    procedure SetLogTop(Value : Double);
    procedure SetLogRight(Value : Double);
    procedure SetLogBottom(Value : Double);
    procedure SetLogCentreX(Value : Double);
    procedure SetLogCentreY(Value : Double);
    procedure SetHorZoom(Value : Double);
    procedure SetVerZoom(Value : Double);
  protected
    { Protected declarations }
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton;  Shift: TShiftState;
       X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton;  Shift: TShiftState;
       X, Y: Integer); override;
    Procedure Zoom; virtual;
    procedure DrawImage;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Canvas;
    procedure DrawData(AGraphar : PArray; const XMin,XMax,YMin,YMax : Extended);
    procedure SetDefaultData;
    function IsPosInImage(APosImageX, APosImageY : Integer): Boolean;
    function PosImageToPosLog(APosImageX, APosImageY : Integer;
              var XAx, YAx: Extended): Boolean;
    property ImageHeight : Integer read FImageHeight;
    property ImageLeft   : Integer read FImageLeft;
    property ImageTop    : Integer read FImageTop;
    property ImageWidth  : Integer read FImageWidth;
    property LogBottom : Double read FLogBottom write SetLogBottom;
    property LogLeft : Double read FLogLeft write SetLogLeft;
    property LogRight : Double read FLogRight write SetLogRight;
    property LogTop : Double read FLogTop write SetLogTop;
    property LogCentreX : Double read FLogCentreX write SetLogCentreX;
    property LogCentreY : Double read FLogCentreY write SetLogCentreY;
    property HorZoom : Double read FHorZoom write SetHorZoom;
    property VerZoom : Double read FVerZoom write SetVerZoom;
    property GraphSizeX : Integer read NumberY;
    property GraphSizeY : Integer read NumberX;
    procedure SetSizeGraph(AGraphSizeX, AGraphSizeY : Integer;
                 const XMin,XMax,YMin,YMax : Double);
{    function GetGraph(X, Y: Integer): Double;
    procedure SetGraph(X, Y: Integer; Value: Double); }
    property Graph[X, Y: Integer]: Double read GetGraph write SetGraph;
    procedure ZoomArea(const ALeft,ATop,ARight,ABottom : Extended);
    procedure ZoomAt(const X,Y : Extended; const AHor,AVer : Extended);
    procedure ZoomAtPos(const X,Y : Extended);
    procedure ZoomAtTime(const AHor,AVer : Extended);
    procedure ZoomOut;
  published
    { Published declarations }
    property About : String read FAbout write SetAbout;
    property Align;
    property Anchors;
    property Axes : TAxes read FAxes write SetAxes;    
    property Color;
    property ColorFon : TColor read FColorFon write SetColorFon default clWhite;
    property ColorGrid : TColor read FColorGrid write SetColorGrid default clBlack;
    property ColorImageFon : TColor read FColorImageFon write SetColorImageFon default clWhite;
    property ColorMap : TColorMaps read FColorMap write SetColorMap default cmSpectr;
    property ColorMinValue : TColor read FColorMinValue write SetColorMinValue default clBlack;
    property ColorMaxValue : TColor read FColorMaxValue write SetColorMaxValue default clWhite;
    property Constraints;
    property ContourLines :Integer read FContourLines write SetContourLines default 10;
    property DragCursor;
    property DragKind;
    property DragMode;
    property DrawFacet : boolean read FDrawFacet write SetDrawFacet default True;
    property Enabled;
    property Font;
    property MouseOn : Boolean read FMouseOn write SetMouseOn;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Title : TTitle read FTitle write SetTitle;
    property TypeImage : TDrawTypes read FTypeImage write SetTypeImage default dtFlat;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown : TMouseEvent read FOnMouseDown write FOnMouseDown;
    property OnMouseMove : TMouseMoveEvent read FOnMouseMove write FOnMouseMove;
    property OnMouseUp : TMouseEvent read FOnMouseUp write FOnMouseUp;
    property OnPaint : TNotifyEvent read FOnPaint write FOnPaint;
    property OnStartDock;
    property OnStartDrag;
    property OnZoom: TNotifyEvent read FOnZoom write FOnZoom;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TPcolor]);
end;

{ TDIBWork }

constructor TDIBWork.PaletteCreate;
var
   dc : HDC;
begin
  inherited Create;
  dc := GetDC(0);
  try
    GetSystemPaletteEntries(dc,0,256,LogPaletteDIB.palEntry);
  finally
    ReleaseDC(0,DC);
  end;
  with LogPaletteDIB do
  begin
    palVersion    := $300;
    palNumEntries := 256;
  end;
  FHandlePal := CreatePalette(pLogPalette(@LogPaletteDIB)^);
end;

constructor TDIBWork.Create;
begin
  inherited Create;
  FBits   := nil;
  FHandle := CreateCompatibleDC(0);
  TDIBWork.PaletteCreate;
  FOldPalette := SelectPalette(FHandle,FHandlePal,false);
  with BitmapInfo.bmiHeader do
  begin
    biSize          := SizeOf(TBITMAPINFOHEADER);
    biPlanes        := 1;
    biBitCount      := 8;
    biCompression   := BI_RGB;
    biWidth         := AWidth;
    biHeight        := AHeigth;
    biSizeImage     := 0;
    biXPelsPerMeter := 0;
    biYPelsPerMeter := 0;
    biClrUsed       := 0;
    biClrImportant  := 0;
  end;
  FWidth := BitmapInfo.bmiHeader.biWidth;
  FHeight := BitmapInfo.bmiHeader.biHeight;
  OldBitmap  := 0;
  DWordWidth := ((aWidth+3) shr 2)shl 2;
  hDIB := CreateDIBSection( FHandle,
                            pBitmapInfo(@BitmapInfo)^,
                            DIB_PAL_COLORS,
                            FBits,
                            0,0);
  OldBitmap := SelectObject(FHandle, hDIB);
end;

destructor TDIBWork.Destroy;
begin
  if OldBitmap <> 0 then
    SelectObject(FHandle, OLdBitmap);
  if FOldPalette <> 0 then
    SelectObject(FHandle, FOldPalette);
  if hDIB <> 0 then
    DeleteObject(hDIB);
  if FHandlePal<>0 then DeleteObject(FHandlePal);
  DeleteDC(FHandle);
  inherited Destroy;
end;

procedure TDIBWork.PaletteChange(LogPal : TLogPaletteDIB);
begin
  LogPal.palVersion    := $300;
  LogPal.palNumEntries := 256;
  LogPaletteDIB := LogPal;
  FHandlePal := CreatePalette(pLogPalette(@LogPal)^);
end;

procedure TDIBWork.SetPalette(LogPal : TLogPaletteDIB);
var
   RGBQuads : array[0..255] of TRGBQuad;
   i : integer;
begin
  if FOldPalette<>0      then SelectPalette(FHandle,FOldPalette,false);
  PaletteChange(LogPal);
  with LogPal do
  for i:=0 to 255 do
  begin
    RGBQuads[i].rgbRed      := palEntry[i].peRed;
    RGBQuads[i].rgbGreen    := palEntry[i].peGreen;
    RGBQuads[i].rgbBlue     := palEntry[i].peBlue;
    RGBQuads[i].rgbReserved := 0;
  end;

  SetDIBColorTable(FHandle,0,256,RGBQuads);
  FOldPalette := SelectPalette(FHandle,FHandlePal,false);
end;

procedure TDIBWork.SetPixel(X, Y: Integer; Value: byte);
begin
//  if (x < FWidth) and (x>=0) and (y<FHeight) and (y>=0) then
     pByte ( integer(FBits) + y*DWordWidth + x )^ := Value;
end;

function  TDIBWork.GetPixel(X, Y: Integer): byte;
begin
//  if (x < FWidth) and (x>=0) and (y<FHeight) and (y>=0) then
     Result := pByte ( integer(FBits) + y*DWordWidth + x )^;
//  else
//     Result :=0;
end;

procedure TDIBWork.Paint(destDC : hDC; ALeft,ATop : Integer);
begin
  SelectPalette(destDC,FHandlePal,false);
  BitBlt(destDC,ALeft,ATop,FWidth,FHeight,FHandle,0,0,SRCCOPY);
end;

procedure TDIBWork.Resize(Width,Height : integer);
begin
  DWordWidth := ((Width+3) shr 2)shl 2;
  with BitmapInfo.bmiHeader do
  begin
    biWidth  := Width;
    biHeight := Height;
  end;
  FWidth := BitmapInfo.bmiHeader.biWidth;
  FHeight := BitmapInfo.bmiHeader.biHeight; 
  if OldBitmap <> 0 then
    SelectObject(FHandle, OldBitmap);
  if hDIB <> 0 then
    DeleteObject(hDIB);
  hDIB := CreateDIBSection( FHandle,
                            pBitmapInfo(@BitmapInfo)^,
                            DIB_PAL_COLORS,
                            FBits,
                            0,0);
  OldBitmap := SelectObject(FHandle, hDIB);
end;    

{ End TDIBWork }

{ TTitle }

constructor TTitle.Create;
begin
  inherited Create;
  FFont := TFont.Create;
  FAlignment := taCenter;
  FVisible := True;
end;

destructor TTitle.Destroy;
begin
  FFont.Free;
  inherited Destroy;
end;

procedure TTitle.Assign(Value: TPersistent);
begin
  inherited Assign(Value);
  Alignment := TTitle(Value).Alignment;
  Font := TTitle(Value).Font;
  Text := TTitle(Value).Text;
  Visible := TTitle(Value).Visible;
end;

procedure TTitle.Changed;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TTitle.SetAlignment(Value: TAlignment);
begin
  if Value <> FAlignment then
  begin
    FAlignment := Value;
    Changed;
  end;
end;

procedure TTitle.SetFont(Value: TFont);
begin
  if Value <> FFont then
  begin
    FFont.Assign(Value);
    Changed;
  end;
end;

procedure TTitle.SetText(const Value: TCaption);
begin
  if Value <> FText then
  begin
    FText := Value;
    Changed;
  end;
end;

procedure TTitle.SetVisible(Value: Boolean);
begin
  if Value <> FVisible then
  begin
    FVisible := Value;
    Changed;
  end;
end;

{ End TTitle }

{ TAxes }

constructor TAxes.Create;
begin
  inherited Create;
  FFont := TFont.Create;
  FVisible := True;
  FXMax := 100;
  FXMin := 0;
  FYMax := 100;
  FYMin := 0;
end;

destructor TAxes.Destroy;
begin
  FFont.Free;
  inherited Destroy;
end;

procedure TAxes.Assign(Value: TPersistent);
begin
  inherited Assign(Value);
  Font := TAxes(Value).Font;
  Visible := TAxes(Value).Visible;
  FXMax := TAxes(Value).XMax;
  FXMin := TAxes(Value).XMin;
  FYMax := TAxes(Value).YMax;
  FYMin := TAxes(Value).YMin;
end;

procedure TAxes.Changed;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TAxes.SetFont(Value: TFont);
begin
  if Value <> FFont then
  begin
    FFont.Assign(Value);
    Changed;
  end;
end;

procedure TAxes.SetVisible(Value: Boolean);
begin
  if Value <> FVisible then
  begin
    FVisible := Value;
    Changed;
  end;
end;

{ End TAxes }

{ TPcolor }

constructor TPcolor.Create(AOwner: TComponent);
var
   i,j : integer;
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  Width := 150;
  Height := 150;
  FAbout := 'PColor 1.3. Filatkin Andrey, Saransk';
  FTitle := TTitle.Create;
  FTitle.OnChange := TitleChanged;
  FTitle.Font.OnChange := TitleChanged;
  FTitle.FText := 'Title PColor';
  TitleLeft := LeftMargin;

  FAxes := TAxes.Create;
  FLogLeft := FAxes.XMin;
  FLogTop := FAxes.YMax;
  FLogRight := FAxes.XMax;
  FLogBottom := FAxes.YMin;
  FLogCentreX := (FLogLeft + FLogRight) / 2;
  FLogCentreY := (FLogTop + FLogBottom) / 2;
  FHorZoom := 1;
  FVerZoom := 1;

  FColorFon := clWhite;
  NumberX := 0;
  NumberY := 0;
  FContourLines :=10;
  FColorImageFon := clBtnFace;
  FColorMap := cmSpectr;
  FColorMaxValue := clWhite;
  FDrawFacet := True;
  FMouseOn := True;
  Drawing := False;
  ChangeGraph := True;
  if csDesigning in ComponentState then
  begin
    FTypeImage := dtFlat;
    setlength(FGraph,10,10);
    NumberX := 10;
    NumberY := 10;
    for i:=0 to 9 do
      for j:=0 to 9 do
        FGraph[i,j] := (9-i)*(9-i)+j*j;
  end;
  FDIB := TDIBWork.Create(2,2);
  Sizes;
end;

destructor TPcolor.Destroy;
begin
  FDIB.Free;
  FTitle.Free;
  inherited Destroy;
end;

procedure TPcolor.Sizes;
var
  DC: HDC;
  Rect1: TRect;
begin
  if not (csReading in ComponentState) then
  begin
    FPCWidth := Width;
    FPCHeight := Height;
    TitleWidth := Width - TitleLeft - RightMargin;
    Rect1 := Rect(TitleLeft,TitleTop,TitleWidth+TitleLeft,TitleHeight+TitleTop);
    DC := GetDC(0);
    Canvas.Handle := DC;
    DoDrawText(Rect1, (DT_EXPANDTABS or DT_CALCRECT or DT_WORDBREAK));
    Canvas.Handle := 0;
    ReleaseDC(0, DC);
    TitleTop := abs(round(0.5*FTitle.Font.Height));
    TitleHeight := Rect1.Bottom - Rect1.Top;
    if FTitle.Visible and (FTitle.Text<>'') then TitleVis := True
    else TitleVis := False;
    FImageLeft := LeftMargin;
    FImageWidth := Width - FImageLeft - RightMargin;
    if TitleVis then FImageTop := TitleHeight + 2*TitleTop
    else FImageTop := TopMargin;
    FImageHeight := Height - FImageTop - BottomMargin;
    if (FImageWidth<0) or (FImageHeight<0) then
    begin
      FImageWidth := 0;
      FImageWidth := 0;
    end;
    if (FImageWidth<>FDIB.Width) or (FImageHeight<>FDIB.Height) then
    begin
      FDIB.Resize(FImageWidth,FImageHeight);
      CreatePaletta;
      CreateImage;
    end;
    if Height < (TitleHeight + TitleTop) then TitleVis := False;
  end;
end;

procedure TPcolor.Paint;
begin
  Canvas.Font := Font;
  Canvas.Brush.Color := Color;
  if (FPCWidth<>Width) or (FPCHeight<>Height) then Sizes;
  if ChangeGraph then CreateImage;
  DrawImage;
  if Assigned(FOnPaint) then FOnPaint(Self);
end;

procedure TPcolor.DrawImage;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Rect1 : TRect;    
  DrawStyle: Longint;
  i : integer;
  XStep, YStep, XL, XR, YT, YB : double;

begin
  TempCursor := Screen.Cursor;
  Screen.Cursor := crHourGlass;
  with Canvas do
  begin
    Brush.Style := bsSolid;
    Brush.Color := FColorFon;
    FillRect(Rect(0, 0, Width, Height));
    if (NumberX <> 0) and (NumberY <> 0) then
    begin
      FDIB.Paint(Handle,FImageLeft,FImageTop);
      if FDrawFacet then
      begin
        XStep := HorZoom * (FImageWidth-1)/(NumberX - 1);
        YStep := VerZoom * (FImageHeight-1)/(NumberY - 1);
        XL := LogXToArr(LogLeft);
        XR := LogXToArr(LogRight);
        YT := LogYToArr(LogTop);
        YB := LogYToArr(LogBottom);
        Canvas.Pen.Width:=1;
        Canvas.Pen.Style:=psSolid;
        Canvas.Pen.Mode := pmCopy;
        Canvas.Pen.Color:=FColorGrid;
        for i := (maxint-trunc(maxint-XL)) to trunc(XR) do
          begin
            Canvas.MoveTo(FImageLeft+round((i-XL)*XStep),FImageTop);
            Canvas.LineTo(FImageLeft+round((i-XL)*XStep),FImageTop+FImageHeight);
          end;
        for i := (maxint-trunc(maxint-YT)) to trunc(YB) do
          begin
            Canvas.MoveTo(FImageLeft,FImageTop+round((i-YT)*YStep));
            Canvas.LineTo(FImageLeft+FImageWidth,FImageTop+round((i-YT)*YStep));
          end;
      end;
    end
    else
    begin
      Brush.Color := FColorImageFon;
      FillRect(Rect(FImageLeft, FImageTop, FImageWidth+FImageLeft, FImageHeight+FImageTop));
    end;
    if TitleVis then
    begin
      Brush.Style := bsClear;
      Rect1 := Rect(TitleLeft,TitleTop,TitleWidth+TitleLeft,TitleHeight+TitleTop);
      DrawStyle := DT_EXPANDTABS or DT_WORDBREAK or Alignments[FTitle.Alignment];
      DoDrawText(Rect1, DrawStyle);
    end;
  end;
  Screen.Cursor := TempCursor;
end;

procedure TPcolor.DrawData(AGraphar : PArray; const XMin,XMax,YMin,YMax : Extended);
var
   i,j,NX,NY : integer;
begin
  NX := high(AGraphar^)+1;
  NY := high (AGraphar^[0])+1;

  if (NX = 0)or(NY = 0) then
    SetDefaultData
  else
  begin
    ChangeGraph := True;
    if NX = 1 then NumberX := 2 else NumberX := NX;
    if NY = 1 then NumberY := 2 else NumberY := NY;
    FGraph := nil;
    SetLength(FGraph,NumberY,NumberX);
    for i:=0 to NY-1 do
      for j:=0 to NX-1 do
        FGraph[i,j] := AGraphar^[j, NY - 1 - i];
    if NX = 1 then
      for i:=0 to NumberY-1 do
        FGraph[i,1] := FGraph[i,0];
    if NY = 1 then
      for i:=0 to NumberX-1 do
        FGraph[1,i] := FGraph[0,i];

    if (XMin<>FAxes.XMin) or (XMax<>FAxes.XMax)
        or (YMin<>FAxes.YMin) or (YMax<>FAxes.YMax) then
    begin
      if XMin < XMax then
        begin  FAxes.FXMin := XMin;   FAxes.FXMax := XMax;  end
      else
        begin  FAxes.FXMin := XMax;   FAxes.FXMax := XMin;  end;
      if XMin = XMax then
        if XMin > 0 then FAxes.FXMax := XMin * 1.01
        else if XMin < 0 then FAxes.FXMax := XMin * 0.99
        else FAxes.FXMax := 0.01;
      if YMin < YMax then
        begin  FAxes.FYMin := YMin;   FAxes.FYMax := YMax;  end
      else
        begin  FAxes.FYMin := YMax;   FAxes.FYMax := YMin;  end;
      if YMin = YMax then
        if YMin > 0 then FAxes.FYMax := YMin * 1.01
        else if YMin < 0 then FAxes.FYMax := YMin * 0.99
        else FAxes.FYMax := 0.01;
      ZoomOut;
    end
    else
    begin
      CreateImage;
      Paint; 
    end;
  end;
end;

function TPcolor.GetGraph(X, Y: Integer): Double;
begin
  GetGraph := FGraph[GraphSizeY - 1 - Y, X];
end;

procedure TPcolor.SetGraph(X, Y: Integer; Value: Double);
begin
  FGraph[GraphSizeY - 1 - Y, X] := Value;
  ChangeGraph := True;
end;

procedure TPcolor.SetSizeGraph(AGraphSizeX, AGraphSizeY : Integer;const XMin,XMax,YMin,YMax : Double);
begin
  ChangeGraph := True;
  if AGraphSizeX < 2 then NumberX := 2 else NumberX := AGraphSizeX;
  if AGraphSizeY < 2 then NumberY := 2 else NumberY := AGraphSizeY;
  SetLength(FGraph,NumberY,NumberX);
  if (XMin<>FAxes.XMin) or (XMax<>FAxes.XMax)
    or (YMin<>FAxes.YMin) or (YMax<>FAxes.YMax) then
  begin
    if XMin < XMax then
      begin  FAxes.FXMin := XMin;   FAxes.FXMax := XMax;  end
    else
      begin  FAxes.FXMin := XMax;   FAxes.FXMax := XMin;  end;
    if XMin = XMax then
      if XMin > 0 then FAxes.FXMax := XMin * 1.01
      else if XMin < 0 then FAxes.FXMax := XMin * 0.99
      else FAxes.FXMax := 0.01;
    if YMin < YMax then
      begin  FAxes.FYMin := YMin;   FAxes.FYMax := YMax;  end
    else
      begin  FAxes.FYMin := YMax;   FAxes.FYMax := YMin;  end;
    if YMin = YMax then
      if YMin > 0 then FAxes.FYMax := YMin * 1.01
      else if YMin < 0 then FAxes.FYMax := YMin * 0.99
      else FAxes.FYMax := 0.01;
    ZoomOut;
  end
end;

procedure TPcolor.CreateImage;
begin
  TempCursor := Screen.Cursor;
  Screen.Cursor := crHourGlass;
  ChangeGraph := False;
  case FTypeImage of
    dtFlat    : CreateFlat;
    dtInterp  : CreateInterp;
    dtContour : CreateContour;
  end;
  Screen.Cursor := TempCursor;
end;

procedure TPcolor.CreateFlat;
var
   MaxValGraph, MinValGraph, TempVal : double;
   i,j,t : integer;
   XStep, YStep, XL, XR, YT, YB : double;
begin
  if (NumberX <> 0) and (NumberY <> 0) then
  begin
    MaxValGraph:=FGraph[0,0];
    MinValGraph:=FGraph[0,0];
    for i:=0 to NumberY-1 do
      for j:=0 to NumberX-1 do
        begin
          TempVal := FGraph[i,j];
          if (MaxValGraph < TempVal) then MaxValGraph := TempVal;
          if (MinValGraph > TempVal) then MinValGraph := TempVal;
        end;
    if MaxValGraph = MinValGraph then MaxValGraph := MinValGraph + 0.01;

    XStep := HorZoom * FImageWidth / (NumberX - 1);
    YStep := VerZoom * FImageHeight / (NumberY - 1);
    XL := LogXToArr(LogLeft);
    XR := LogXToArr(LogRight);
    YT := LogYToArr(LogTop);
    YB := LogYToArr(LogBottom);

    With Canvas do
    begin
      Brush.Style:= bsSolid;
      for i:=round(YT) to round(YB) do
        for j:=round(XL) to round(XR) do
        begin
          t := round((FGraph[i,j]-MinValGraph)*(NumbCol-1)/(MaxValGraph-MinValGraph));
          Brush.Color:=FLogPalette.palEntry[t].peRed or (FLogPalette.palEntry[t].peGreen shl 8) or (FLogPalette.palEntry[t].peBlue shl 16);
          Windows.FillRect(FDIB.Handle, Rect(round((j-0.5-XL)*XStep),round((i-0.5-YT)*YStep),round((j+0.5-XL)*XStep),round((i+0.5-YT)*YStep)), Brush.Handle);
        end;
    end;
  end;
end;

procedure TPcolor.CreateInterp;
var
   MaxValGraph, MinValGraph, TempVal : double;
   XStep, YStep, YAver1, YAver2, XYAver, XL, YT  : double;
   i,j,y1,y2,x1,x2,y2pics,x2pics : integer;
begin
  if (NumberX <> 0) and (NumberY <> 0) then
  begin
    MaxValGraph:=FGraph[0,0];
    MinValGraph:=FGraph[0,0];
    for i:=0 to NumberY-1 do
      for j:=0 to NumberX-1 do
        begin
          TempVal := FGraph[i,j];
          if (MaxValGraph < TempVal) then MaxValGraph := TempVal;
          if (MinValGraph > TempVal) then MinValGraph := TempVal;
        end;
    if MaxValGraph = MinValGraph then MaxValGraph := MinValGraph + 0.01;
    
    XL := LogXToArr(LogLeft);
    YT := LogYToArr(LogTop);
    XStep := HorZoom * FImageWidth / (NumberX - 1);
    YStep := VerZoom * FImageHeight / (NumberY - 1);
    y1 := trunc(YT);
    y2 := y1 + 1;
    y2pics := trunc((y2 - YT) * YStep);
    for i:=0 to FImageHeight-1 do
    begin
      x1:=trunc(XL);
      x2:=x1 + 1;
      x2pics := trunc((x2 - XL) * XStep);
      if i>y2pics then
      begin
        y1:=trunc(YT + i/YStep);
        y2:=y1+1;
        y2pics:=trunc((y2 - YT)*YStep);
      end;
      YAver1:=(i-(y1-YT)*YStep)*(FGraph[y2,x1]-FGraph[y1,x1])/YStep+FGraph[y1,x1];
      YAver2:=(i-(y1-YT)*YStep)*(FGraph[y2,x2]-FGraph[y1,x2])/YStep+FGraph[y1,x2];
      for j:=0 to FImageWidth-1 do
      begin
        if j>x2pics then
        begin
          x1 := trunc(XL + j/XStep);
          x2 := x1 + 1;
          x2pics := trunc((x2 - XL) * XStep);
          YAver1:=(i-(y1-YT)*YStep)*(FGraph[y2,x1]-FGraph[y1,x1])/YStep+FGraph[y1,x1];
          YAver2:=(i-(y1-YT)*YStep)*(FGraph[y2,x2]-FGraph[y1,x2])/YStep+FGraph[y1,x2];
        end;
        XYAver := (j-(x1-XL)*XStep)*(YAver2-YAver1)/XStep+YAver1;
        FDIB.Pixels[j,FImageHeight-i-1]:=round((XYAver-MinValGraph)*(NumbCol-1)/(MaxValGraph-MinValGraph));
      end;
    end;
  end;
end;

procedure TPcolor.CreateContour;
var
   MaxValGraph, MinValGraph, TempVal : double;
   MaxValGraphC, MinValGraphC : double;
   XStep,YStep,YAver11,YAver12,YAver21,YAver22: double;
   a11,a12,a21,a22,Max3,Min3 : double;
   hh, XL, YT : double;
   Colors : array of byte;    
   y11,y12,y12pics,y21,y22,y22pics,x21,x22,x22pics,temp : integer;
   i,j,qn : integer;
begin
  if (NumberX <> 0) and (NumberY <> 0) then
  begin
    MaxValGraph:=FGraph[0,0];
    MinValGraph:=FGraph[0,0];
    for i:=0 to NumberY-1 do
      for j:=0 to NumberX-1 do
        begin
          TempVal := FGraph[i,j];
          if (MaxValGraph < TempVal) then MaxValGraph := TempVal;
          if (MinValGraph > TempVal) then MinValGraph := TempVal;
        end;
    if MaxValGraph = MinValGraph then MaxValGraph := MinValGraph + 0.01;
    TempVal:=(MaxValGraph-MinValGraph)*0.01;
    MaxValGraphC := MaxValGraph+TempVal;
    MinValGraphC := MinValGraph-TempVal;

    SetLength(Colors,FContourLines);
    if FContourLines=1 then
      Colors[0]:=round((NumbCol-1)/2)
    else
      for i:=0 to FContourLines-1 do
        Colors[i]:=round((NumbCol-1)*i/(FContourLines-1)); 

    XL := LogXToArr(LogLeft);
    YT := LogYToArr(LogTop);
    XStep := HorZoom * FImageWidth / (NumberX -1);
    YStep := VerZoom * FImageHeight / (NumberY -1);
    hh := (MaxValGraphC-MinValGraphC)/(FContourLines + 1);

    y11 := trunc(YT);
    y12 := y11 + 1;
    y12pics:=trunc((y12 - YT) * YStep);
    y21 := trunc(YT);
    y22 := y21 + 1;
    y22pics:=trunc((y22 - YT) * YStep);
    for i:=0 to FImageHeight-1 do
    begin
      x21:=trunc(XL);
      x22:=x21 + 1;
      x22pics:=trunc((x22 - XL) * XStep);
      if i>y12pics then
      begin
        y11:=trunc(YT + i/YStep);
        y12:=y11+1;
        y12pics:=trunc((y12 - YT) * YStep);
      end;
      if i+1>y22pics then
        begin
          temp:=trunc(YT + (i+1)/YStep);
          if temp < (NumberY - 1) then
            begin
             y21:=temp;
             y22:=y21+1;
             y22pics:=trunc((y22 - YT) * YStep);
            end;
        end;
      YAver11:=(i-(y11-YT)*YStep)*(FGraph[y12,x21]-FGraph[y11,x21])/YStep+FGraph[y11,x21];
      YAver12:=(i-(y11-YT)*YStep)*(FGraph[y12,x22]-FGraph[y11,x22])/YStep+FGraph[y11,x22];
      YAver21:=(i+1-(y21-YT)*YStep)*(FGraph[y22,x21]-FGraph[y21,x21])/YStep+FGraph[y21,x21];
      YAver22:=(i+1-(y21-YT)*YStep)*(FGraph[y22,x22]-FGraph[y21,x22])/YStep+FGraph[y21,x22];
      a12 := (0-(x21-XL)*XStep)*(YAver12-YAver11)/XStep+YAver11;
      a22 := (0-(x21-XL)*XStep)*(YAver22-YAver21)/XStep+YAver21;
      for j:=1 to FImageWidth do
      begin
        if j>x22pics then
        begin
          if x22 < (NumberX - 1) then
          begin
            x21:=trunc(XL + j/XStep);
            x22:=x21+1;
            x22pics:=trunc((x22 - XL) * XStep);
            YAver11:=(i-(y11-YT)*YStep)*(FGraph[y12,x21]-FGraph[y11,x21])/YStep+FGraph[y11,x21];
            YAver12:=(i-(y11-YT)*YStep)*(FGraph[y12,x22]-FGraph[y11,x22])/YStep+FGraph[y11,x22];
            YAver21:=(i+1-(y21-YT)*YStep)*(FGraph[y22,x21]-FGraph[y21,x21])/YStep+FGraph[y21,x21];
            YAver22:=(i+1-(y21-YT)*YStep)*(FGraph[y22,x22]-FGraph[y21,x22])/YStep+FGraph[y21,x22];
          end;
        end;
        a11 := a12;
        a12 := (j-(x21-XL)*XStep)*(YAver12-YAver11)/XStep+YAver11;
        a21 := a22;
        a22 := (j-(x21-XL)*XStep)*(YAver22-YAver21)/XStep+YAver21;
        Max3:=a11;  Min3:=a11;
        if a12>Max3 then Max3:=a12;
        if a12<Min3 then Min3:=a12;
        if a21>Max3 then Max3:=a21;
        if a21<Min3 then Min3:=a21;
        if a22>Max3 then Max3:=a22;
        if a22<Min3 then Min3:=a22;
        qn:=trunc((Max3-MinValGraphC)/hh);
        if qn>=(maxint-trunc(maxint-(Min3-MinValGraphC)/hh)) then
          FDIB.Pixels[j-1,FImageHeight-i-1]:=Colors[qn-1]
        else FDIB.Pixels[j-1,FImageHeight-i-1]:=NumbCol;
      end;
    end;
  end;
end;

procedure TPcolor.SetAbout;
begin
end;

procedure TPcolor.SetDefaultData;
begin
  NumberX := 0;
  NumberY := 0;
  SetLength(FGraph,NumberY,NumberX);
  ZoomOut;
  Paint;
end;

procedure TPcolor.SetDrawFacet(ADrawFacet : boolean);
begin
  FDrawFacet := ADrawFacet;
  Paint;
end;

procedure TPcolor.SetColorGrid(AColorGrid : TColor);
begin
  FColorGrid := AColorGrid;
  Paint;
end;

procedure TPcolor.SetColorFon(AColor : TColor);
begin
  FColorFon := AColor;
  Invalidate;
end;

procedure TPcolor.SetColorImageFon(AColor : TColor);
begin
  FColorImageFon := AColor;
  CreatePaletta;
  Invalidate;
end;

procedure TPcolor.SetColorMap(AColorMap : TColorMaps);
begin
  FColorMap := AColorMap;
  CreatePaletta;
  Paint;   
end;

procedure TPcolor.SetColorMaxValue(AColor : TColor);
begin
  FColorMaxValue := AColor;
  if FColorMap=cmLine then
  begin
    CreatePaletta;
    Paint;
  end;  
end;

procedure TPcolor.SetColorMinValue(AColor : TColor);
begin
  FColorMinValue := AColor;
  if FColorMap=cmLine then
  begin
    CreatePaletta;
    Paint;
  end;  
end;

procedure TPcolor.SetContourLines(AContourLines : Integer);
begin
  if AContourLines<2 then FContourLines:=1
  else FContourLines := AContourLines;
  if FTypeImage=dtContour then
  begin
    CreateContour;
    Paint;
  end;  
end;

procedure TPcolor.SetTypeImage(ATypeImage : TDrawTypes);
begin
  if (FTypeImage<>ATypeImage) then
  begin
    FTypeImage := ATypeImage;
    CreateImage;
    Paint;
  end;
end;

procedure TPcolor.CreatePaletta;
var
   CMinRed,CMinGreen,CMinBlue : byte;
   CMaxRed,CMaxGreen,CMaxBlue : byte;
   i : integer;
   x:array[0..15] of byte;
   NC8 : byte;
begin
  case FColorMap of
   cmLine :
    begin
      CMinRed := ColorToRGB(FColorMinValue) and $000000FF;
      CMinGreen := (ColorToRGB(FColorMinValue) and $0000FF00) shr 8;
      CMinBlue := (ColorToRGB(FColorMinValue) and $00FF0000) shr 16;
      CMaxRed := ColorToRGB(FColorMaxValue) and $000000FF;
      CMaxGreen := (ColorToRGB(FColorMaxValue) and $0000FF00) shr 8;
      CMaxBlue := (ColorToRGB(FColorMaxValue) and $00FF0000) shr 16;
      for i:=0 to NumbCol-1 do
        begin
          FLogPalette.palEntry[i].peRed := round(i*(CMaxRed-CMinRed)/(NumbCol-1))+CMinRed;
          FLogPalette.palEntry[i].peGreen := round(i*(CMaxGreen-CMinGreen)/(NumbCol-1))+CMinGreen;
          FLogPalette.palEntry[i].peBlue := round(i*(CMaxBlue-CMinBlue)/(NumbCol-1))+CMinBlue;
          FLogPalette.palEntry[i].peFlags := pc_Reserved;
        end;
    end;
   cmSpectr :
    begin
      NC8 := round(NumbCol/8);
      for i:=0 to NC8*2-1 do x[i]:=round(255*(i+1)/(NC8*2));
      for i:=    0 to NC8*1-1 do
        begin
          FLogPalette.palEntry[i].peRed   := 0;
          FLogPalette.palEntry[i].peGreen := 0;
          FLogPalette.palEntry[i].peBlue  := x[i+NC8];
          FLogPalette.palEntry[i].peFlags := pc_Reserved;
        end;
      for i:=NC8*1 to NC8*3-1 do
        begin
          FLogPalette.palEntry[i].peRed   := 0;
          FLogPalette.palEntry[i].peGreen := x[i-NC8];
          FLogPalette.palEntry[i].peBlue  := 255;
          FLogPalette.palEntry[i].peFlags := pc_Reserved;
        end;
      for i:=NC8*3 to NC8*5-1 do
        begin
          FLogPalette.palEntry[i].peRed   := x[i-NC8*3];
          FLogPalette.palEntry[i].peGreen := 255;
          FLogPalette.palEntry[i].peBlue  := x[NC8*5-1-i];
          FLogPalette.palEntry[i].peFlags := pc_Reserved;
        end;
      for i:=NC8*5 to NC8*7-1 do
        begin
          FLogPalette.palEntry[i].peRed   := 255;
          FLogPalette.palEntry[i].peGreen := x[NC8*7-1-i];
          FLogPalette.palEntry[i].peBlue  := 0;
          FLogPalette.palEntry[i].peFlags := pc_Reserved;
        end;
      for i:=NC8*7 to NC8*8-1 do
        begin
          FLogPalette.palEntry[i].peRed   := x[NC8*9-1-i];
          FLogPalette.palEntry[i].peGreen := 0;
          FLogPalette.palEntry[i].peBlue  := 0;
          FLogPalette.palEntry[i].peFlags := pc_Reserved;
        end;
    end;
   cmRandom :
    begin
      Randomize;
      for i:=0 to NumbCol-1 do
        begin
          FLogPalette.palEntry[i].peRed   := Random(255);
          FLogPalette.palEntry[i].peGreen := Random(255);
          FLogPalette.palEntry[i].peBlue  := Random(255);
          FLogPalette.palEntry[i].peFlags := pc_Reserved;
        end;
    end;
  end;
  i:=NumbCol;
  FLogPalette.palEntry[i].peRed := ColorToRGB(FColorImageFon) and $000000FF;
  FLogPalette.palEntry[i].peGreen := (ColorToRGB(FColorImageFon) and $0000FF00) shr 8;
  FLogPalette.palEntry[i].peBlue := (ColorToRGB(FColorImageFon) and $00FF0000) shr 16;
  FLogPalette.palEntry[i].peFlags := pc_Reserved;
  FDIB.SetPalette(FLogPalette);
end;

function TPColor.IsPosInImage(APosImageX, APosImageY : Integer): Boolean;
begin
  if (APosImageX < FImageLeft) or (APosImageY < FImageTop)
     or (APosImageX > FImageLeft + FImageWidth - 1)
     or (APosImageY > FImageTop + FImageHeight - 1) then Result := False
  else Result := True;
end;

function TPColor.PosImageToPosLog(APosImageX, APosImageY : Integer;
            var XAx, YAx: Extended): Boolean;
begin
  Result := IsPosInImage(APosImageX, APosImageY);
  if (APosImageX < FImageLeft) then APosImageX := FImageLeft;
  if (APosImageY < FImageTop) then APosImageY := FImageTop;
  if (APosImageX > FImageLeft + FImageWidth - 1) then
    APosImageX := FImageLeft + FImageWidth - 1;
  if (APosImageY > FImageTop + FImageHeight - 1) then
    APosImageY := FImageTop + FImageHeight - 1;
  if FImageWidth = 1 then XAx := Axes.XMin
  else XAx := LogLeft+(LogRight-LogLeft)*(APosImageX-FImageLeft)/(FImageWidth-1);
  if FImageHeight = 1 then YAx := Axes.YMin
  else YAx := LogTop-(LogTop-LogBottom)*(APosImageY-FImageTop)/(FImageHeight-1);
end;

procedure TPColor.TitleChanged(Sender: TObject);
begin
  Sizes;
  Paint;
end;

procedure TPcolor.SetTitle(Value: TTitle);
begin
  FTitle.Assign(Value);
end;

procedure TPcolor.DoDrawText(var Rect: TRect; Flags: Longint);
var
  Text : String;
begin
  Text := FTitle.Text;
  if (Flags and DT_CALCRECT <> 0) and (Text = '') then Text := ' ';
  Flags := Flags or DT_NOPREFIX;
  Canvas.Font := FTitle.Font;
  DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
end;

procedure TPcolor.SetAxes(Value: TAxes);
begin
  FAxes.Assign(Value);
end;

procedure TPcolor.Zoom;
begin
  if Assigned(FOnZoom) then FOnZoom(Self);
end;

function TPColor.LogXToArr(Value : Double) : Double;
begin
  Result:=((Value-FAxes.XMin)/(FAxes.XMax-FAxes.XMin))*(NumberX-1);
end;

function TPColor.LogYToArr(Value : Double) : Double;
begin
  Result:=((FAxes.YMax-Value)/(FAxes.YMax-FAxes.YMin))*(NumberY-1);
end;

procedure TPColor.ZoomAt(const X,Y : Extended; const AHor,AVer : Extended);
var
  Xt,Yt,Hor,Ver,XC,YC : Extended;
begin
  Hor := Abs(AHor);
  Ver := Abs(AVer);
  XC := X;
  YC := Y;
  if Hor < 1 then Hor := 1;
  if Hor > MaxZoom then Hor := MaxZoom;
  if Ver < 1 then Ver := 1;
  if Ver > MaxZoom then Ver := MaxZoom;
  Xt := (FAxes.XMax - FAxes.XMin) / (2 * Hor);
  Yt := (FAxes.YMax - FAxes.YMin) / (2 * Ver);
  if (XC - Xt) < FAxes.XMin then XC := FAxes.XMin + Xt;
  if (XC + Xt) > FAxes.XMax then XC := FAxes.XMax - Xt;
  if (YC - Yt) < FAxes.YMin then YC := FAxes.YMin + Yt;
  if (YC + Yt) > FAxes.YMax then YC := FAxes.YMax - Yt;
  if (Hor <> FHorZoom) or (Ver <> FVerZoom)
      or ((XC - Xt) <> FLogLeft) or ((XC + Xt) <> FLogRight)
      or ((YC - Yt) <> FLogBottom) or ((YC + Yt) <> FLogTop) then
  begin
    FHorZoom := Hor;
    FVerZoom := Ver;
    FLogCentreX := XC;
    FLogCentreY := YC;
    FLogLeft := XC - Xt;
    FLogRight := XC + Xt;
    FLogTop := YC + Yt;
    FLogBottom := YC - Yt;
    CreateImage;
    Paint;
    Zoom;
  end;  
end;

procedure TPColor.ZoomAtPos(const X,Y : Extended);
begin
  ZoomAt(X,Y,HorZoom,VerZoom);
end;

procedure TPColor.SetLogCentreX(Value : Double);
begin
  ZoomAt(Value,LogCentreY,HorZoom,VerZoom);
end;

procedure TPColor.SetLogCentreY(Value : Double);
begin
  ZoomAt(LogCentreX,Value,HorZoom,VerZoom);
end;

procedure TPColor.ZoomAtTime(const AHor,AVer : Extended);
begin
  ZoomAt(LogCentreX,LogCentreY,AHor,AVer);
end;

procedure TPColor.SetHorZoom(Value : Double);
begin
  ZoomAt(LogCentreX,LogCentreY,Value,VerZoom);
end;

procedure TPColor.SetVerZoom(Value : Double);
begin
  ZoomAt(LogCentreX,LogCentreY,HorZoom,Value);
end;
   
procedure TPColor.ZoomOut;
begin
  ZoomAtTime(1,1);
end;

procedure TPColor.ZoomArea(const ALeft,ATop,ARight,ABottom : Extended);
var
  Hor,Ver,AL,AT,AR,AB : Extended;
begin
  if ALeft < FAxes.XMin then AL := FAxes.XMin
   else if ALeft > FAxes.XMax then AL := FAxes.XMax
        else AL := ALeft;
  if ARight > FAxes.XMax then AR := FAxes.XMax
   else if ARight < FAxes.XMin then AR := FAxes.XMin
        else AR := ARight;
  if ATop > FAxes.YMax then AT := FAxes.YMax
   else if ATop < FAxes.YMin then AT := FAxes.YMin
        else AT := ATop;
  if ABottom < FAxes.YMin then AB := FAxes.YMin
   else if ABottom > FAxes.YMax then AB := FAxes.YMax
        else AB := ABottom;
  if AR = AL then Hor := MaxZoom
   else Hor := (FAxes.XMax - FAxes.XMin)/(AR - AL);
  if AT = AB then Ver := MaxZoom
   else Ver := (FAxes.YMax - FAxes.YMin)/(AT - AB);
  ZoomAt((ALeft+ARight)/2, (ATop+ABottom)/2, Hor, Ver);
end;

procedure TPColor.SetLogLeft(Value : Double);
begin
  ZoomArea(Value,LogTop,LogRight,LogBottom);
end;

procedure TPColor.SetLogTop(Value : Double);
begin
  ZoomArea(LogLeft,Value,LogRight,LogBottom);
end;

procedure TPColor.SetLogRight(Value : Double);
begin
  ZoomArea(LogLeft,LogTop,Value,LogBottom);
end;

procedure TPColor.SetLogBottom(Value : Double);
begin
  ZoomArea(LogLeft,LogTop,LogRight,Value);
end;

procedure TPColor.SetMouseOn(const Value: Boolean);
begin
  FMouseOn := Value;
end;

procedure TPColor.MouseDown(Button: TMouseButton; Shift: TShiftState;
   X, Y: Integer);
begin
  if (NumberX <> 0) and (NumberY <> 0) and
     MouseOn and IsPosInImage(X, Y) then
  begin
    Drawing := True;
    Origin := Point(X, Y);
    MovePt := Origin;
  end;  
  if Assigned(FOnMouseDown) then FOnMouseDown(Self, Button, Shift, X, Y);
end;

procedure TPColor.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  if Drawing and (NumberX <> 0) and (NumberY <> 0) and
     MouseOn and IsPosInImage(X, Y) then
  begin
    Canvas.Pen.Color := clBlack;
    Canvas.Pen.Style := psDot;
    Canvas.Pen.Mode := pmNotXor;
    Canvas.Rectangle(Origin.X, Origin.Y, MovePt.X, MovePt.Y);
    MovePt := Point(X, Y);
    Canvas.Rectangle(Origin.X, Origin.Y, MovePt.X, MovePt.Y);
  end;
  if Assigned(FOnMouseMove) then FOnMouseMove(Self, Shift, X, Y);
end;


procedure TPColor.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  XA, YA, X1, Y1 : Extended;
begin
  if (NumberX <> 0) and (NumberY <> 0) and MouseOn then
  begin
   Drawing := False;
   if (Origin.X <> MovePt.X) or (Origin.Y <> MovePt.Y) then
   begin
     Canvas.Pen.Color := clBlack;
     Canvas.Pen.Style := psDot;
     Canvas.Pen.Mode := pmNotXor;
     Canvas.Rectangle(Origin.X, Origin.Y, MovePt.X, MovePt.Y);
     if (Origin.X <> MovePt.X) and (Origin.Y <> MovePt.Y) then
     begin
       PosImageToPosLog(Origin.x, Origin.y, X1, Y1);
       PosImageToPosLog(MovePt.x, MovePt.y, XA, YA);
       ZoomArea(X1, Y1, XA, YA);
     end;
   end
   else if PosImageToPosLog(X, Y, XA, YA) then
     if (ssShift in Shift) and (Button = mbLeft) then ZoomAtPos(XA, YA)
     else if (Shift=[]) then
          if (Button = mbLeft) then ZoomAt(XA, YA, HorZoom * 2, VerZoom * 2)
          else if (Button = mbRight) then ZoomAt(XA, YA, HorZoom/2, VerZoom/2);
  end;       
  if Assigned(FOnMouseUp) then FOnMouseUp(Self, Button, Shift, X, Y);
end;

{ End TPColor }

end.
